## run the following commands through a unix terminal to compile the C
## code (from within the appropriate directory):
##
## R CMD SHLIB distances.c getListElement.c
## R CMD SHLIB mate_probs.c getListElement.c
## R CMD SHLIB comp_coeffs.c getListElement.c

## ***************************************************************************
## ************* run/show a simulation ***************
## ***************************************************
## the below path needs to be changed appropriately
setwd("~/Desktop/coexist")
rm(list=ls())
source("src_r/initialize.R")
## ***************************************************************************
## Gaussian functions for local carrying capacity
G.1D <- function(x,y)
  gaussian.1D(x, y, sigma=0.1, p.h=c(1,1), K.base=3.560631)
G.2D <- function(x,y)
  gaussian.2D(x, y, sigma=0.1, p.h=c(1,1), K.base=3.560631)
## create a list of parameters
prms <- prms.f(dimension = as.integer(2),
               spatial.model = as.integer(2), ## 1=no spatial variation, 2=spatial variation
               genetic.model = as.integer(1), ## 1=allelic, 2=quantitative
               comp.model = as.integer(1), ## 1=survival, 2=fecundity
               num.loci = as.integer(1), ## number of loci (not used or tested)
               move.model = as.integer(1), ## 1=constant, 2=mating dependent dispersal
               asymmetric.costs = 0, ## 0=no asymmetric costs, 1=asummetric costs
               num.types = as.integer(2), ## number of mating types
               sigma.s = 0.05, ## competition radius
               sigma.f = 0.05, ## female search radius
               sigma.m = 0.05, ## movement radius
               sigma.p = 0.1, ## preference width for quant model
               sigma.r = 0.01, ## mutational width for quant model
               recomb.rate = 0.5, ## recombination rate
               N = as.integer(1000), ## pre-competition pop size
               K = as.integer(500), ## integral of k(x,y)
               alpha = 5, ## preference strength
               fem.cost = 500, ## strength of mate-search costs
               cost.type.1 = 0, ## asymmetric cost strength
               s0 = 1, ## free survival parameter
               x.len = 1, ## length of spatial x-dimesion
               y.len = 0.5, ## length of spatial y-dimesion
               fxy=G.2D, ## function specifying spatial variation in local carrying capacity
               male.move = c(1,100), ## mating-dependent dispersal parameters
               num.gens = as.integer(100), ## number of generations to run the simulation for
               init.spatial.config = as.integer(2), ## 1=uniform
               print.every = as.integer(1), ## refresh rate when plotting
               file.name = "res.RData")
## this runs the simulation
quartz(width=7, height=2)
res <- run.sim(prms, TRUE, brk.if.fix=FALSE)
