#include <R.h> 
#include <Rdefines.h>
#include <Rmath.h> 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* compute competition coefficents for all pairs of individuals that
   are within crit_d of one another */
void compute_comp_coeffs(double *dist_d, int dim, int *ind,
			double *comp_coeffs, double *sums,
			int n, int N, double sigma_s) {
  int i;
  double num;
  double denom;

  if(dim==1) denom = sqrt(2 * PI) * sigma_s;
  else denom = 2 * PI * sigma_s * sigma_s;

  /* first compute all competition coefficients for the corresponding
     pairs of distances that are stored in the sparse matrix */
  for( i=0; i<n; i++ ) {
    num = dist_d[i] / sigma_s;
    comp_coeffs[i] = exp( -0.5 * num * num ) / denom;
  }

  /* make a vector to store the summed competition coefficients for
     each of the 'N' individuals */
  for(i=0; i<N; i++) {
    sums[i]=0;
  }

  /* sum up the competition coefficients for each individual - note
     that IND contains the list of all i and j indices from the sparse
     matrix (e.g., IND=c(d$i, d$j)) and thus this sum must go from 0
     to 2*n-1 (or equivalently, 1 to 2*n) so that each competition
     coefficient is added to both sums for individuals i and j */
  for( i=0; i<2*n; i++ ) {
    /* minus one so indexing is as in C */
    sums[ind[i]-1] = sums[ind[i]-1] + comp_coeffs[i%n];
  }

}

/* construct sparse matrix of pairwise distances */
SEXP comp_coeffs(SEXP DIM, SEXP DIST, SEXP IND, SEXP N, SEXP SIGMA_S) {
  int n = LENGTH(DIST);
  int dim = *INTEGER(DIM);
  SEXP comp_coeffs, sums, ret;

  PROTECT(comp_coeffs = allocVector(REALSXP, n));
  PROTECT(sums = allocVector(REALSXP, *INTEGER(N)));

  compute_comp_coeffs(REAL(DIST),
		      dim,
		      INTEGER(IND),
		      REAL(comp_coeffs),
		      REAL(sums),
		      n,
		      *INTEGER(N),
		      *REAL(SIGMA_S));

  PROTECT(ret = allocVector(VECSXP, 1));
  SET_VECTOR_ELT(ret, 0, sums);
  UNPROTECT(3);
  return ret;
}
