## ******************************************************************
## ********************** everyone gives birth **********************
## ******************************************************************

## figure out the mating pairs of males and females
mating.pairs <- function(prms, pop, distances) {
  comp.sums <- sum.comp.coeffs(prms, pop, distances)
  ## survival model
  if(prms$comp.model==1)
    a.i <- rep(1, pop$pop.size)
  ## fecundity model
  if(prms$comp.model==2)
    a.i <- compute.resource.shares(prms, pop, comp.sums)

  ## mating probabilities
  mate.probs <- c_mate.probs(prms, pop, distances,
                             availabilities = a.i*(pop$sex==0))

  split.index <- split(seq_along(mate.probs$i), mate.probs$i)
  females <- as.integer(names(split.index))
  males <- lapply(split.index, function(x) mate.probs$j[x])
  probs <- lapply(split.index, function(x) mate.probs$mate.prob[x])
  
  ## female birth rates
  birth.rates <- a.i[females]/(1 + prms$fem.cost/sapply(probs, sum))
  
  choose.dad <- function(p)
    rmultinom(1, 1, p) > 0
  
  ## number of offspring per female (constant population size of N)
  num <- as.vector(rmultinom(1, size=prms$N, prob=birth.rates))

##   ## number of offspring per female (population size of approx N)
##   beta <- 9
##   num <- rpois(length(females), birth.rates*beta)
  
  ok <- which(num>0)
  mothers <- rep(females[ok], num[ok])
  fathers <- rep(sapply(ok, function(x)
                        males[[x]][choose.dad(probs[[x]])]), num[ok])
  list(mothers=mothers, fathers=fathers)
}

## create offspring
birth <- function(prms, pop, distances) {
  ## decide who mates with who
  mp <- mating.pairs(prms, pop, distances)
  num.offspring <- length(mp$mothers)
  
  ## allelic model
  ##
  if(prms$genetic.model==1) {
    ## free recombination
    if(prms$recomb.rate==0.5) {
      f.disp <- runif(num.offspring*prms$num.loci)<0.5
      f.pref <- runif(num.offspring*prms$num.loci)<0.5
    }
    ## complete linkage
    if(prms$recomb.rate==0) {
      f.disp <- runif(num.offspring*prms$num.loci)<0.5
      f.pref <- f.disp
    }
    ## partial recombination
    if(prms$recomb.rate!=0 & prms$recomb.rate!=0.5) {
      f.disp <- runif(num.offspring*prms$num.loci)<0.5
      tmp <- rep(prms$recomb.rate, num.offspring*prms$num.loci)
      tmp[f.disp] <- 1-prms$recomb.rate
      f.pref <- runif(num.offspring*prms$num.loci)<tmp
    }

    ## f.disp (f.pref) individuals get display (preference) allele of
    ## their father.
    disp <- pop$disp[mp$mothers,]
    pref <- pop$pref[mp$mothers,]
    disp[f.disp] <- pop$disp[mp$fathers,][f.disp]
    pref[f.pref] <- pop$pref[mp$fathers,][f.pref]
  }
  
  ## quantitative model
  ##
  ## offspring trait values are drawn from a Gaussian with mean equal
  ## to the mid-parernt value, and standard deviation sigma.r
  if(prms$genetic.model==2) {
    if(prms$num.loci>1) cat("Error, num.loci should be set to 1\n")
    disp.means <- (pop$disp[mp$mothers,] + pop$disp[mp$fathers,])/2
    pref.means <- (pop$pref[mp$mothers,] + pop$pref[mp$fathers,])/2
    disp <- rnorm(length(disp.means), disp.means, prms$sigma.r)
    pref <- rnorm(length(pref.means), pref.means, prms$sigma.r)
  }
  
  list(x=pop$x[mp$mothers],
       y=pop$y[mp$mothers],
       sex=as.integer(runif(num.offspring)<0.5),
       disp=matrix(disp, ncol=prms$num.loci),
       pref=matrix(pref, ncol=prms$num.loci),
       pop.size=as.integer(num.offspring),
       mating.pop.size=pop$pop.size)
}
