## R functions that call the C functions using the .Call interface

## ******************************************************************
## ********************** Distances matrix **************************
## ******************************************************************
c_distances <- function(prms, pop) {
  d <- .Call("distances",
             pop$x,
             pop$y,
             as.numeric(prms$x.len),
             as.numeric(prms$y.len),
             as.integer(prms$init.sparse.dim),
             as.numeric(prms$crit.distance))
  names(d) <- c("i", "j", "distance", "size")
  d$i <- d$i[seq_len(d$size)]
  d$j <- d$j[seq_len(d$size)]
  d$distance <- d$distance[seq_len(d$size)]
  d
}

## ******************************************************************
## **************** Compute competition coefficients ****************
## ******************************************************************
c_comp.coeffs <- function(prms, d, pop.size)
  .Call("comp_coeffs",
        as.integer(prms$dimension),
        as.numeric(d$distance),
        as.integer(c(d$i, d$j)),
        as.integer(pop.size),
        as.numeric(prms$sigma.s))[[1]]

## ******************************************************************
## *************** return mating propensities matrix ****************
## ******************************************************************
c_mate.probs <- function(prms, pop, distances, availabilities) {
  mp <- .Call("mate_probs",
              prms,
              pop,
              distances,
              availabilities)
  names(mp) <- c("i", "j", "mate.prob", "size")
  mp$i <- mp$i[seq_len(mp$size)]
  mp$j <- mp$j[seq_len(mp$size)]
  mp$mate.prob <- mp$mate.prob[seq_len(mp$size)]
  mp
}
