## function to check if the population is fixed at the display
## (preference) for the trait specified
disp.fixation <- function(pop, trait)
  sum(abs(diff(pop$disp[,trait])))==0

pref.fixation <- function(pop, trait)
  sum(abs(diff(pop$pref[,trait])))==0

## iterate a single generation
next.gen <- function(prms, pop) {
  distances <- c_distances(prms, pop)
  if(distances$size > prms$init.sparse.dim)
    prms$init.sparse.dim <- as.integer(distances$size)
  ## mating and birth (includes competition)
  if(prms$comp.model==1) {
    pop <- death(prms, pop, distances)
    distances <- c_distances(prms, pop)
  }
  pop <- birth(prms, pop, distances)
  ## move and compute new distances
  pop <- move(prms, pop)
  pop
}

## Run the simulation
run.sim <- function(prms, plot, slp=0, brk.if.fix=TRUE, print.out=TRUE) {
  pop <- initialize.pop(prms)
  res <- list()
  for(i in 1:prms$num.gens) {
    Sys.sleep(slp)
    if(i%%prms$print.every==0) {
      mean.disp <- mean(pop$disp)
      mean.pref <- mean(pop$pref)
    }
    if(print.out && i%%prms$print.every==0) {
      cat(sprintf("%s, ", prms$file.name))
      cat(sprintf("Generation: %d, ", i))
      cat(sprintf("%d types, ", length(unique(pop$disp))))
      cat(sprintf("disp/pref=%2.3f/%2.3f\n", mean.disp, mean.pref))
    }
    ## next generation
    pop <- next.gen(prms, pop)
    ## plot every 'print.every' generations
    if(i%%prms$print.every==0) {
      res[[i/prms$print.every]] <- pop
      if(plot) {
        plot.sexual.traits(prms, pop, 1)
      }
    }
    ## check for fixation
    if(brk.if.fix)
      if(disp.fixation(pop, 1) || pref.fixation(pop, 1)) {
        save(res, file=prms$file.name)
        break
      }
        
  }
  save(res, file=prms$file.name)
  res
}
