## ******************************************************************
## ********************** move each individual **********************
## ******************************************************************
move.distances <- function(prms, pop)
  rnorm(pop$pop.size, mean=0, sd=prms$sigma.m)

## Mating dependent dispersal
move.distances.PK <- function(prms, pop) {
  ## all pair-wise distances
  distances <- c_distances(prms, pop)

  ## compute mate.probs the usual way
  if(prms$comp.model==1)
    a.i <- rep(1, pop$pop.size)
  ## fecundity model
  if(prms$comp.model==2)
    a.i <- compute.resource.shares(prms, pop, comp.sums)
  ## mating probabilities
  mate.probs <- c_mate.probs(prms, pop, distances,
                             availabilities = a.i*(pop$sex==0))

  ## compute sums in Eq. 12
  males <-   which(pop$sex==0)
  sum.i.pij <- sapply(males, function(x)
                      sum(mate.probs$mate.prob[mate.probs$j==x]))
  sum.i.k.pik <- sum(mate.probs$mate.prob)
  ## compute fraction in Eq. 12
  sp <- sum.i.pij / sum.i.k.pik
  
  ## scale movement sd for males
  sd.list <- rep(1, pop$pop.size)
  sd.list[males] <- prms$male.move[1]*exp(-prms$male.move[2]*sp)
  sd.list <- sd.list*prms$sigma.m
  rnorm(pop$pop.size, mean=0, sd=sd.list)
} 

move <- function(prms, pop) {
  if(prms$move.model==1)
    d <- move.distances(prms, pop)
  if(prms$move.model==2)
    d <- move.distances.PK(prms, pop)
  if(prms$dimension==1) {
    pop$x <- (pop$x + d)%%1
  } else {
    a = runif(pop$pop.size) * 2*pi
    pop$x <- (pop$x + cos(a) * d)%%prms$x.len
    pop$y <- (pop$y + sin(a) * d)%%prms$y.len
  }
  pop
}
