## ********** Plot sexual traits coloured in time **********
library(RColorBrewer)
plot.sexual.traits <- function(prms, d, trait) {
  ## range of trait values observed
  trait.range <- range(c(d$disp, d$pref))
  plot.pop <- function(trait.vals, s, gen) {
    ## add wiggle to y values so it is easier to see in 1D
    if(prms$dimension==1)
      d$y <- d$y + runif(length(d$y))/4-0.125

    cols <- (trait.vals-trait.range[1])/diff(trait.range)
    ## *** allelic model
    if(prms$genetic.model==1) {
      if(prms$num.types==2)
        cols.dis <- brewer.pal(5, "Spectral")[c(1,5)][trait.vals+1]
      if(prms$num.types>2)
        cols.dis <- (1:prms$num.types)[trait.vals+1]
      plot(d$x/prms$x.len, d$y/prms$y.len, xlim=c(0,1), ylim=c(0,1),
           xlab = "", ylab = "", yaxt = "n", xaxt = "n", pch=20,
           col=cols.dis, cex = 0.75)
    }
    ## *** quantitative model
    if(prms$genetic.model==2) {
      cols.cts <- rgb(cols,1-cols,0)
      plot(d$x/prms$x.len, d$y/prms$y.len, xlim=c(0,1), ylim=c(0,1),
           xlab = "", ylab = "", yaxt = "n", xaxt = "n", pch=20,
           col=cols.cts, cex = 0.75)
      t1 <- sprintf("= %2.3f", max(trait.range))
      t2 <- sprintf("= %2.3f", min(trait.range))
      m1 <- max(cols.cts)
      m2 <- min(cols.cts)
      legend.text <- c(t1, t2)
      legend("bottomleft", legend.text, col=c(m1, m2),
             pch=c(20,20), bg="white", inset = 0.02, x.intersp=0.5,
             y.intersp=0.7)
    }
    axis(1, at=c(0:3)/3, yaxt = "n", xaxt = "n", labels=FALSE)
    axis(2, at=c(0:2)/2, yaxt = "n", xaxt = "n", labels=FALSE)
    title(main=s)
  }
  par(mfrow=c(1, 2), mar=c(.5, 1, 1, .5))
  plot.pop(d$disp[,trait], "Display Trait", i)
  plot.pop(d$pref[,trait], "Preference Trait", i)
}
## *********************************************************
